@@ -1,3 +1,27 @@
+/*
+ * Dropbear - a SSH2 server
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
 #include "options.h"
 #include "packet.h"
 #include "buffer.h"
@@ -8,6 +32,7 @@
 #include "sshpty.h"
 #include "termcodes.h"
 #include "ssh.h"
+#include "random.h"
 
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 		char iscmd);
@@ -76,7 +101,7 @@ static void sesssigchild_handler(int val) {
 
 		/* find the corresponding chansess */
 		for (i = 0; i < ses.childpidsize; i++) {
-			assert(pid > 1); /* XXX */
+			assert(pid > 1);
 			if (ses.childpids[i].pid == pid) {
 
 				assert(ses.childpids[i].chansess != NULL);
@@ -209,7 +234,6 @@ void closechansess(struct Channel *channel) {
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].chansess == chansess) {
 			assert(ses.childpids[i].pid > 0);
-			/* XXX kill the process? or will it die of natural causes? */
 			TRACE(("closing pid %d\n", ses.childpids[i].pid));
 			TRACE(("exited = %d\n", chansess->exited));
 			ses.childpids[i].pid = -1;
@@ -240,7 +264,7 @@ void chansessionrequest(struct Channel *channel) {
 	if (typelen > MAX_NAME_LEN) {
 		send_msg_channel_failure(channel);
 		m_free(type);
-		TRACE(("leave chansessionrequest: type too long")); /* XXX error? */
+		TRACE(("leave chansessionrequest: type too long")); /* XXX send error?*/
 		return;
 	}
 
@@ -662,36 +686,55 @@ static void execchild(struct ChanSess *chansess) {
 
 	/* wipe the hostkey */
 	sign_key_free(ses.opts->hostkey);
-	/* close file descriptors except stdin/stdout/stderr */
+
+	/* clear the state of the prng */
+	seedrandom();
+
+	/* close file descriptors except stdin/stdout/stderr
+	 * Need to be sure FDs are closed here to avoid reading files as root */
 	for (i = 3; i < ses.maxfd; i++) {
-		close(i);
+		if (m_close(i) == -1) {
+			dropbear_exit("Error closing file desc");
+		}
 	}
+
+
 	/* clear environment */
 	/* if we're debugging using valgrind etc, we need to keep the LD_PRELOAD
-	 * etc. this is hazardous, so should only be used for debugging. */
+	 * etc. This is hazardous, so should only be used for debugging. */
 #ifndef DEBUG_KEEP_ENV
 #ifdef HAVE_CLEARENV
 	clearenv();
 #else /* don't HAVE_CLEARENV */
-	/*environ = NULL; - won't work */
+	environ = (char**)m_malloc(ENV_SIZE * sizeof(char*));
+	environ[0] = NULL;
 #endif /* HAVE_CLEARENV */
-#endif
+#endif /* DEBUG_KEEP_ENV */
 
-	/* change user */
-	if (setgid(ses.authstate.pw->pw_gid) < 0) {
-		dropbear_exit("error changing user");
-	}
-#ifndef HACKCRYPT
-	/* this will fail if we aren't root - check? XXX*/
-	if (initgroups(ses.authstate.pw->pw_name,
+	/* We can only change uid/gid as root ... */
+	if (getuid() == 0) {
+
+		if (setgid(ses.authstate.pw->pw_gid) < 0
+			|| initgroups(ses.authstate.pw->pw_name,
 				ses.authstate.pw->pw_gid) < 0) {
-		dropbear_exit("error changing user");
-	}
-#endif
-	if (setuid(ses.authstate.pw->pw_uid) < 0) {
-		dropbear_exit("error changing user");
-	}
+			dropbear_exit("error changing user");
+		}
 
+		if (setuid(ses.authstate.pw->pw_uid) < 0) {
+			dropbear_exit("error changing user");
+		}
+	} else {
+		/* ... but if the daemon is the same uid as the requested uid, we don't
+		 * need to */
+
+		/* XXX - there is a minor issue here, in that if there are multiple
+		 * usernames with the same uid, but differing groups, then the
+		 * differing groups won't be set (as with initgroups()). The solution
+		 * is for the sysadmin not to give out the UID twice */
+		if (getuid() != ses.authstate.pw->pw_uid) { dropbear_exit("couldn't
+				change user as non-root");
+		}
+	}
 
 	/* set env vars */
 	addnewvar("USER", ses.authstate.pw->pw_name);
