@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -62,7 +62,7 @@
 #define DROPBEAR_RSA
 #define DROPBEAR_DSS
 
-/* Define DSS_PROTOK to use Putty's method of generating the value k for dss,
+/* Define DSS_PROTOK to use PuTTY's method of generating the value k for dss,
  * rather than just from the random byte source.
  * Undefining this will save you ~4k in binary size with static uclibc, but
  * your DSS hostkey could be exposed if the random number source isn't good.
@@ -75,11 +75,11 @@
 #define DROPBEAR_PUBKEY_AUTH
 
 /* Random device to use - you must specify one only.
- * DEV_RANDOM is recommended on hosts with a good /dev/random,
+ * DEV_RANDOM is recommended on hosts with a good /dev/urandom,
  * otherwise use EGD and run EGD or PRNGD, specifying
  * the socket. This is only used for the initial seed, further
  * entropy is gathered from timings etc */
-#define DROPBEAR_DEV_RANDOM /* use /dev/random and /dev/urandom */
+#define DROPBEAR_DEV_URANDOM /* use /dev/urandom */
 
 /*#undef DROPBEAR_EGD */ /* use egd or prngd socket */
 #define DROPBEAR_EGD_SOCKET "./rng"
@@ -95,7 +95,7 @@
  * You shouldn't edit below here unless you know you need to.
  *******************************************************************/
 
-#define DROPBEAR_VERSION "0.28"
+#define DROPBEAR_VERSION "0.30"
 #define LOCAL_IDENT "SSH-2.0-dropbear_" DROPBEAR_VERSION
 #define PROGNAME "dropbear"
 
@@ -110,6 +110,10 @@
 /* Close connections to clients which haven't authorised after AUTH_TIMEOUT */
 #define AUTH_TIMEOUT 600 /* 10 minutes is recommended */
 
+/* success/failure defines */
+#define DROPBEAR_SUCCESS 0
+#define DROPBEAR_FAILURE -1
+
 /* various algorithm identifiers */
 #define DROPBEAR_KEX_DH_GROUP1 0
 
@@ -157,7 +161,7 @@
 
 #define MAX_BANNER_SIZE 2000 /* this is 25*80 chars, any more is foolish */
 
-#define DEV_RANDOM "/dev/urandom"
+#define DEV_URANDOM "/dev/urandom"
 
 /* the number of NAME=VALUE pairs to malloc for environ, if we don't have
  * the clearenv() function */
