@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -42,6 +42,8 @@
 #include "libtomcrypt/mycrypt.h"
 
 static void read_packet_init();
+static void process_postauth_packet(unsigned int type);
+static void recv_unimplemented();
 
 #define ZLIB_COMPRESS_INCR 20 /* this is 12 bytes + 0.1% of 8000 bytes */
 #define ZLIB_DECOMPRESS_INCR 100
@@ -415,6 +417,60 @@ void process_packet() {
 	/* handle the packet depending on type */
 	ses.expecting = 0;
 
+	switch (type) {
+
+		case SSH_MSG_SERVICE_REQUEST:
+			recv_msg_service_request();
+			break;
+
+		case SSH_MSG_USERAUTH_REQUEST:
+			recv_msg_userauth_request();
+			break;
+			
+		case SSH_MSG_KEXINIT:
+			recv_msg_kexinit();
+			break;
+
+		case SSH_MSG_KEXDH_INIT:
+			recv_msg_kexdh_init();
+			break;
+
+		case SSH_MSG_NEWKEYS:
+			recv_msg_newkeys();
+			break;
+
+		case SSH_MSG_CHANNEL_DATA:
+		case SSH_MSG_CHANNEL_WINDOW_ADJUST:
+		case SSH_MSG_CHANNEL_REQUEST:
+		case SSH_MSG_CHANNEL_OPEN:
+		case SSH_MSG_CHANNEL_EOF:
+		case SSH_MSG_CHANNEL_CLOSE:
+			/* these should be checked for authdone below */
+			process_postauth_packet(type);
+			break;
+	
+		default:
+			/* TODO this possibly should be handled */
+			TRACE(("unknown packet"));
+			recv_unimplemented();
+			break;
+	}
+
+out:
+	buf_free(ses.payload);
+	ses.payload = NULL;
+
+	TRACE(("leave process_packet"));
+}
+
+/* process a packet, and also check that auth has been done */
+static void process_postauth_packet(unsigned int type) {
+
+	/* messages following here require userauth before use */
+	if (!ses.authstate.authdone) {
+		dropbear_exit("received message %d before userauth", type);
+	}
+
 	switch (type) {
 
 		case SSH_MSG_CHANNEL_DATA:
@@ -440,39 +496,36 @@ void process_packet() {
 		case SSH_MSG_CHANNEL_CLOSE:
 			recv_msg_channel_close();
 			break;
-
-		case SSH_MSG_SERVICE_REQUEST:
-			recv_msg_service_request();
-			break;
-
-		case SSH_MSG_USERAUTH_REQUEST:
-			recv_msg_userauth_request();
-			break;
 			
-		case SSH_MSG_KEXINIT:
-			recv_msg_kexinit();
-			break;
-
-		case SSH_MSG_KEXDH_INIT:
-			recv_msg_kexdh_init();
-			break;
-
-		case SSH_MSG_NEWKEYS:
-			recv_msg_newkeys();
-			break;
-
 		default:
-			/* TODO this possibly should be handled */
-			TRACE(("unknown packet"));
+			TRACE(("unknown packet()"));
+			recv_unimplemented();
+			break;
 	}
-
-out:
-	buf_free(ses.payload);
-	ses.payload = NULL;
-
-	TRACE(("leave process_packet"));
 }
 
+/* This must be called directly after receiving the unimplemented packet.
+ * Isn't the most clean implementation, it relies on packet processing
+ * occurring directly after decryption. This is reasonably valid, since
+ * there is only a single decryption buffer */
+static void recv_unimplemented() {
+
+	unsigned int seq;
+
+	/* the decryption routine increments the sequence number, we must
+	 * decrement */
+	seq = ses.recvseq - 1;
+
+	CHECKCLEARTOWRITE();
+
+	buf_putbyte(ses.writepayload, SSH_MSG_UNIMPLEMENTED);
+	buf_putint(ses.writepayload, seq);
+
+	encrypt_packet();
+}
+	
+
+
 /* encrypt the writepayload, putting into writebuf, ready for write_packet()
  * to put on the wire */
 void encrypt_packet() {
