@@ -1,3 +1,27 @@
+/*
+ * Dropbear - a SSH2 server
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
 #include "options.h"
 #include "util.h"
 #include "bignum.h"
@@ -6,7 +30,6 @@
 #include "ssh.h"
 #include "random.h"
 
-#include "libtomcrypt/mpi.h"
 #include "libtomcrypt/mycrypt.h"
 
 #ifdef DROPBEAR_RSA 
@@ -16,7 +39,8 @@ static mp_int * rsa_pad_em(rsa_key * key,
 
 /* Load a rsa key from a buffer, initialising the values.
  * The key will have the same format as buf_put_rsa_key.
- * These should be freed with rsa_key_free */
+ * These should be freed with rsa_key_free.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_get_rsa_pub_key(buffer* buf, rsa_key *key) {
 
 	TRACE(("enter buf_get_rsa_pub_key"));
@@ -29,19 +53,19 @@ int buf_get_rsa_pub_key(buffer* buf, rsa_key *key) {
 
 	buf_incrpos(buf, 4+SSH_SIGNKEY_RSA_LEN); /* int + "ssh-rsa" */
 
-	if (buf_getmpint(buf, key->e) != 0
-	 || buf_getmpint(buf, key->n) != 0) {
-		dropbear_msg("failure reading rsa pubkey");
+	if (buf_getmpint(buf, key->e) == DROPBEAR_FAILURE
+	 || buf_getmpint(buf, key->n) == DROPBEAR_FAILURE) {
 		TRACE(("leave buf_get_rsa_pub_key: failure"));
-		return -1;
+		return DROPBEAR_FAILURE;
 	}
 	TRACE(("leave buf_get_rsa_pub_key: success"));
-	return 0;
+	return DROPBEAR_SUCCESS;
 
 }
 
 /* same as buf_get_rsa_pub_key, but reads a private "x" key at the end.
- * Loads a private rsa key from a buffer */
+ * Loads a private rsa key from a buffer
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_get_rsa_priv_key(buffer* buf, rsa_key *key) {
 
 	int ret;
@@ -50,18 +74,15 @@ int buf_get_rsa_priv_key(buffer* buf, rsa_key *key) {
 	TRACE(("enter buf_get_rsa_priv_key"));
 
 	ret = buf_get_rsa_pub_key(buf, key);
-	if (ret != 0) {
-		goto out;
+	if (ret == DROPBEAR_FAILURE) {
+		TRACE(("leave buf_get_rsa_priv_key: ret == DROPBEAR_FAILURE"));
+		return DROPBEAR_FAILURE;
 	}
 
 	key->d = m_malloc(sizeof(mp_int));
 	m_mp_init(key->d);
 	ret = buf_getmpint(buf, key->d);
-out:
-	if (ret != 0) {
-		dropbear_msg("failure reading rsa privkey");
-	}
-		
+
 	TRACE(("leave buf_get_rsa_priv_key"));
 	return ret;
 }
@@ -125,13 +146,14 @@ void buf_put_rsa_priv_key(buffer* buf, rsa_key *key) {
 }
 
 #ifdef DROPBEAR_SIGNKEY_VERIFY
+/* returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_rsa_verify(buffer * buf, rsa_key *key, const unsigned char* data,
 		unsigned int len) {
 
 	unsigned int slen;
 	mp_int rsa_s, rsa_mdash;
 	mp_int *rsa_em = NULL;
-	int ret = 0;
+	int ret = DROPBEAR_FAILURE;
 
 	assert(key != NULL);
 
@@ -158,7 +180,7 @@ int buf_rsa_verify(buffer * buf, rsa_key *key, const unsigned char* data,
 
 	if (mp_cmp(rsa_em, &rsa_mdash) == 0) {
 		/* signature is valid */
-		ret = 1;
+		ret = DROPBEAR_SUCCESS;
 	}
 
 out:
